package com.fossgalaxy.games.tbs.ai.rules;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.actions.AbstractAction;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.order.Order;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.object.annotations.ObjectDef;
import com.fossgalaxy.object.annotations.ObjectDefStatic;
import org.codetome.hexameter.core.api.CubeCoordinate;

import java.util.Collection;
import java.util.List;
import java.util.Objects;

public class BuildRule extends FirstLegalRule {
    private final EntityType produced;
    private final Integer maxAmount;

    @ObjectDef("build")
    public BuildRule(EntityType produced, Integer maxAmount) {
        super(AbstractAction.CAT_BUILD);

        this.produced = Objects.requireNonNull(produced);
        this.maxAmount = maxAmount;
    }

    @ObjectDef("build")
    public BuildRule(EntityType produced) {
        this(produced, null);
    }

    @ObjectDefStatic("EnsureEntity")
    public static ProductionRule buildEnsureEntity(EntityType producer, EntityType produced, int qty){
        return new RuleWrapper(new BuildRule(produced, qty), producer);
    }


    @Override
    public Order generateOrder(GameState state, Entity entity) {

        //do we have enouph already?
        if (maxAmount != null) {

            Collection<Entity> myTypes = state.getOwnedEntities(entity.getOwner());
            long owned = myTypes.stream().filter(e -> e.getType().isInstance(produced)).count();

            if (owned >= maxAmount) {
                return null;
            }

        }


        //find a space to build it in
        CubeCoordinate coordinate = chooseTarget(entity, state);
        if (coordinate == null) {
            return null;
        }


        //find an action which can build it
        List<GameAction> actions = entity.getType().getAvailableActions();
        for (GameAction action : actions) {

            if (!category.equals(action.getCategory()) || !action.appliesTo(produced)) {
                continue;
            }


            if (action.isPossible(entity, state, coordinate)) {
                return action.generateOrder(coordinate, state);
            }

        }

        return null;
    }

    @Override
    protected CubeCoordinate chooseTarget(Entity entity, GameState state) {
        return AgentUtils.findBuildingPosition(state, produced, entity.getPos());
    }
}
