package rts.ai.evaluation;

import com.fossgalaxy.games.tbs.GameState;
import com.fossgalaxy.games.tbs.actions.GeneratorAction;
import com.fossgalaxy.games.tbs.actions.MineByProperty;
import com.fossgalaxy.games.tbs.actions.MinesAction;
import com.fossgalaxy.games.tbs.entity.Entity;
import com.fossgalaxy.games.tbs.parameters.EntityType;
import com.fossgalaxy.games.tbs.ui.GameAction;
import com.fossgalaxy.object.annotations.ObjectDef;

import java.util.Collection;
import java.util.List;

public class EconomicAndMilitaryFunction extends EvaluationFunction{

    private final float resourceBonus;
    private final float productionBonus;
    private final float unitBonus;

    @ObjectDef("EandM")
    public EconomicAndMilitaryFunction(float resourceBonus, float productionBonus, float unitBonus) {
        this.resourceBonus = resourceBonus;
        this.productionBonus = productionBonus;
        this.unitBonus = unitBonus;
    }

    @Override
    public float evaluate(int maxplayer, int minplayer, GameState gs) {
        float s1 = base_score(maxplayer,gs);
        float s2 = base_score(minplayer,gs);
        if (s1 + s2 == 0) return 0.5f;
        return  (2*s1 / (s1 + s2))-1;
    }

    public float base_score(int player, GameState gs){
        float score = 0f;

        for (String rt : gs.getSettings().getResourceNames()){
            score += gs.getResource(player, rt) * resourceBonus;
        }

        Collection<Entity> myDudes = gs.getOwnedEntities(player);
        if (myDudes.isEmpty()) {
            return 0;
        }

        for (Entity entity : myDudes) {
            double unitCost = 0;
            EntityType t = entity.getType();
            for (int resCost : t.getCosts().values()) {
                unitCost += resCost;
            }

            List<GameAction> actions = entity.getType().getAvailableActions();
            for(GameAction action : actions){
                if(action instanceof GeneratorAction){
                    GeneratorAction generatorAction = (GeneratorAction) action;
                    score += (generatorAction.getQuantityPerTurn() * productionBonus);
                }else if(action instanceof MinesAction){
                    MinesAction minesAction = (MinesAction) action;
                    score += (minesAction.getQuantityPerTurn() * productionBonus);
                }else if(action instanceof MineByProperty){
                    MineByProperty mineByProperty = (MineByProperty) action;
                    score += (entity.getProperty(mineByProperty.getPropertyTag()) * productionBonus);
                }
            }

            score += unitCost * unitBonus * Math.sqrt(entity.getHealthFrac());
        }

        return score;
    }

    @Override
    public float upperBound(GameState gs) {
        return 1;
    }
}
